unit KA.Data.SQL.AdoNet;

interface

uses
  Borland.VCL.SysUtils,
  Borland.VCL.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.ADONETDb,
  System.Data,
  System.Data.SqlClient,
  System.ComponentModel;

type
  TAdoNetDataSetNotifyEvent = procedure(DataSet: TDataSet; Err : String) of object;

  TKASQLAdoNet       = Class;
  TKAADONETConnector = Class(TADONETConnector)
  private
     FKASQLAdoNet : TKASQLAdoNet;
  protected
     procedure DoAfterPost; Override;
     procedure DoAfterDelete; Override;
  End;

  TKASQLAdoNet = class(TComponent)
  private
    FADONETConnector        : TKAADONETConnector;
    FADONETSQLDataSet       : DataSet;
    FADONETSQLConnection    : SqlConnection;
    FADONETSQLAdapter       : SqlDataAdapter;
    FADONETSQLCommand       : SqlCommand;
    FDONETSQLCommandBuilder : SqlCommandBuilder;
    FConnectionString       : String;
    FActive                 : Boolean;
    FTableName              : String;
    FTables                 : TStrings;
    FDataSource             : TDataSource;
    FOnDeleteError          : TAdoNetDataSetNotifyEvent;
    FOnPostError            : TAdoNetDataSetNotifyEvent;
    FSQL                    : TStrings;
    { Private declarations }
  protected
    { Protected declarations }
    procedure SetADONETSQLAdapter(const Value: SqlDataAdapter);
    procedure SetADONETSQLConnection(const Value: SqlConnection);
    procedure SetADONETSQLDataSet(const Value: DataSet);
    procedure SetADONETConnector(const Value: TKAADONETConnector);
    procedure SetConnectionString(const Value: String);
    procedure SetDataSource(const Value: TDataSource);
    procedure SetTableName(const Value: String);
    procedure SetOnDeleteError(const Value: TAdoNetDataSetNotifyEvent);
    procedure SetOnPostError(const Value: TAdoNetDataSetNotifyEvent);
    procedure SetSQL(const Value: TStrings);

    function  GetTables: TStrings;
    procedure SetTables(const Value: TStrings);


    procedure SetActive(const Value: Boolean);
    Procedure DoAfterPost(DataSet: TDataSet);
    Procedure DoAfterDelete(DataSet: TDataSet);
    Procedure Loaded;Override;
    Procedure Notification(AComponent: TComponent; Operation: TOperation);Override;
  public
    { Public declarations }
    Constructor Create(AOwner : TComponent);Override;
    Destructor Destroy;Override;
    Function   PromptEditor(ConnectStr: String): String;
    property   ADONETConnector     : TKAADONETConnector        read FADONETConnector     write SetADONETConnector;
    property   ADONETSQLConnection : SqlConnection             read FADONETSQLConnection write SetADONETSQLConnection;
    property   ADONETSQLAdapter    : SqlDataAdapter            read FADONETSQLAdapter    write SetADONETSQLAdapter;
    property   ADONETSQLDataSet    : DataSet                   read FADONETSQLDataSet    write SetADONETSQLDataSet;
  published
    { Published declarations }
    Property   ConnectionString    : String                    read FConnectionString    write SetConnectionString;
    Property   TableName           : String                    read FTableName           write SetTableName;
    Property   Tables              : TStrings                  read GetTables            write SetTables;
    Property   SQL                 : TStrings                  read FSQL                 write SetSQL;
    Property   DataSource          : TDataSource               read FDataSource          write SetDataSource;
    Property   OnPostError         : TAdoNetDataSetNotifyEvent read FOnPostError         write SetOnPostError;
    Property   OnDeleteError       : TAdoNetDataSetNotifyEvent read FOnDeleteError       write SetOnDeleteError;
    Property   Active              : Boolean                   read FActive              write SetActive;
  end;

procedure Register;

implementation
Uses
   System.Reflection,
   System.Runtime.InteropServices,
   Borland.Vcl.ActiveX,
   Borland.Vcl.ComObj;
{$R 'TKASQLAdoNet.bmp'}

Const
  Class_DataLinks        : TGUID = '{2206CDB2-19C1-11D1-89E0-00C04FD7A829}';
  IID_DataSourceLocator  : TGUID = '{2206CCB2-19C1-11D1-89E0-00C04FD7A829}';
  Class_Connection       : TGUID = '{00000514-0000-0010-8000-00AA006D2EA4}';
  IID_Connection         : TGUID = '{00000550-0000-0010-8000-00AA006D2EA4}';


procedure Register;
begin
  RegisterComponents('KA Ado', [TKASQLAdoNet]);
end;

{ TKASQLAdoNet }

constructor TKASQLAdoNet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FADONETConnector              := TKAADONETConnector.Create(Self);
  FADONETConnector.FKASQLAdoNet := Self;
  FSQL                          := TStringList.Create;
  FTables                       := TStringList.Create;
  FSQL.Add('SELECT * FROM [%s]');
  FADONETSQLDataSet             := Nil;
  FADONETSQLConnection          := Nil;
  FADONETSQLAdapter             := Nil;
  FADONETSQLCommand             := Nil;
  FDONETSQLCommandBuilder       := Nil;
end;

destructor TKASQLAdoNet.Destroy;
begin
  Active := False;
  FTables.Free;
  FSQL.Free;
  inherited Destroy;
end;

Function TKASQLAdoNet.PromptEditor(ConnectStr: String): String;
var
   NewOrEdit : TObject;
   Connect   : TObject;
   Obj       : Array[0..0] of TObject;
Begin
  Result := '';
  ConnectStr := 'Provider=SQLOLEDB;'+ConnectStr;
  OleCheck(CoCreateInstance(CLASS_DataLinks, nil, CLSCTX_ALL, IID_DataSourceLocator, NewOrEdit));
  if Assigned(NewOrEdit) then
     Begin
       if ConnectStr = '' then
          Begin
            Connect := NewOrEdit.GetType.InvokeMember('PromptNew',BindingFlags.InvokeMethod,Nil,NewOrEdit,Nil);
            try
              if Connect <> Nil Then
                 Result := Connect.GetType.InvokeMember('ConnectionString',BindingFlags.GetProperty,Nil,Connect,Nil).ToString
              Else
                 Result := '';
            Except
            End;
          End
       Else
          Begin
            OleCheck(CoCreateInstance(CLASS_Connection, nil, CLSCTX_ALL, IID_Connection, Connect));
            Obj[0] := TObject(ConnectStr);
            Connect.GetType.InvokeMember('ConnectionString',BindingFlags.SetProperty,Nil,Connect,Obj);
            Obj[0] := Connect;
            if Boolean(NewOrEdit.GetType.InvokeMember('PromptEdit',BindingFlags.InvokeMethod,Nil,NewOrEdit,Obj)) Then
               Begin
                 Result  := Connect.GetType.InvokeMember('ConnectionString',BindingFlags.GetProperty,Nil,Connect,Nil).ToString
               End;
          End;
     End;
     if Result <> '' Then Delete(Result,1,Pos(';',Result));
     Connect := nil;
     NewOrEdit := nil;
End;


procedure TKASQLAdoNet.SetADONETConnector(const Value: TKAADONETConnector);
begin
  //***************************************************************** READ ONLY
end;

procedure TKASQLAdoNet.SetADONETSQLAdapter(const Value: SqlDataAdapter);
begin
  //***************************************************************** READ ONLY
end;

procedure TKASQLAdoNet.SetADONETSQLConnection(const Value: SqlConnection);
begin
  //***************************************************************** READ ONLY
end;

procedure TKASQLAdoNet.SetADONETSQLDataSet(const Value: DataSet);
begin
  //***************************************************************** READ ONLY
end;

procedure TKASQLAdoNet.SetConnectionString(const Value: String);
begin
  FConnectionString := Value;
  if csLoading in ComponentState Then Exit;
  Active := False;
end;

procedure TKASQLAdoNet.SetActive(const Value: Boolean);
begin
  if Value Then
     Begin
       if FConnectionString='' Then Raise Exception.Create('ConnectionString property not set!');
       if FTableName=''        Then Raise Exception.Create('TableName property not set!');
     End;
  FActive := Value;
  if FActive Then
     Begin
       if Assigned(FDataSource) Then            FDataSource.DataSet                := Nil;
       FADONETSQLConnection                  := SqlConnection.Create;
       FADONETSQLAdapter                     := SqlDataAdapter.Create;
       FDONETSQLCommandBuilder               := SqlCommandBuilder.Create(ADONETSQLAdapter);
       FADONETSQLDataSet                     := DataSet.Create;
       FADONETSQLConnection.ConnectionString := ConnectionString;
       FADONETSQLConnection.Open;
       FADONETSQLCommand                     := SqlCommand.Create(Format(FSQL.Text,[FTableName]),ADONETSQLConnection);
       FADONETSQLAdapter.SelectCommand       := FADONETSQLCommand;
       FADONETSQLAdapter.DeleteCommand       := FDONETSQLCommandBuilder.GetDeleteCommand;
       FADONETSQLAdapter.InsertCommand       := FDONETSQLCommandBuilder.GetInsertCommand;
       FADONETSQLAdapter.UpdateCommand       := FDONETSQLCommandBuilder.GetUpdateCommand;
       FADONETSQLAdapter.Fill(FADONETSQLDataSet, FTableName);
       FADONETConnector.DataTable            := FADONETSQLDataSet.Tables[FTableName];
       FADONETConnector.Active               := True;
       if Assigned(FDataSource) Then            FDataSource.DataSet                := FADONETConnector;
     End
 Else
    Begin
      FADONETConnector.Active               := False;
      if Assigned(FADONETSQLConnection)    Then FADONETSQLConnection.Close;
      if Assigned(ADONETSQLDataSet)        Then FreeAndNil(FADONETSQLDataSet);
      if Assigned(FDONETSQLCommandBuilder) Then FreeAndNil(FDONETSQLCommandBuilder);
      if Assigned(FADONETSQLCommand)       Then FreeAndNil(FADONETSQLCommand);
      if Assigned(ADONETSQLAdapter)        Then FreeAndNil(FADONETSQLAdapter);
      if Assigned(ADONETSQLConnection)     Then FreeAndNil(FADONETSQLConnection);
    End;
end;

procedure TKASQLAdoNet.SetTableName(const Value: String);
begin
  FTableName := Value;
end;

procedure TKASQLAdoNet.SetDataSource(const Value: TDataSource);
begin
  if Assigned(FDataSource) Then FDataSource.RemoveFreeNotification(Self);
  FDataSource := Value;
  if Assigned(FDataSource) Then FDataSource.FreeNotification(Self);
end;

procedure TKASQLAdoNet.DoAfterDelete(DataSet: TDataSet);
begin
  Try
    FADONETConnector.ApplyUpdates(FADONETSQLAdapter);
  Except
    On E:Exception do
       Begin
         Active := False;
         Active := True;
         if Assigned(FOnDeleteError) Then
            FOnDeleteError(DataSet,E.Message)
         Else
            Raise;
       End;
  End;
end;

procedure TKASQLAdoNet.DoAfterPost(DataSet: TDataSet);
begin
  Try
    FADONETConnector.ApplyUpdates(FADONETSQLAdapter);
  Except
    On E:Exception do
       Begin
         Active := False;
         Active := True;
         if Assigned(FOnPostError) Then
            FOnPostError(DataSet,E.Message)
         Else
            Raise;
       End;
  End;
end;

procedure TKASQLAdoNet.SetOnDeleteError(const Value: TAdoNetDataSetNotifyEvent);
begin
  FOnDeleteError := Value;
end;

procedure TKASQLAdoNet.SetOnPostError(const Value: TAdoNetDataSetNotifyEvent);
begin
  FOnPostError := Value;
end;

procedure TKASQLAdoNet.Loaded;
begin
  inherited Loaded;
  if Assigned(FDataSource) Then
     Begin
       FDataSource.DataSet := Nil;
       FDataSource.DataSet := FADONETConnector;
     End;
end;

procedure TKASQLAdoNet.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  If (Operation = opRemove) And (AComponent = FDataSource) Then FDataSource := Nil;
end;

procedure TKASQLAdoNet.SetSQL(const Value: TStrings);
begin
  FSQL.Assign(Value);
end;

procedure TKASQLAdoNet.SetTables(const Value: TStrings);
begin
  //***************************************************************** READ ONLY
end;

function TKASQLAdoNet.GetTables: TStrings;
Var
  SQLDataSet_    : DataSet;
  SQLConnection_ : SqlConnection;
  SQLAdapter_    : SqlDataAdapter;
  Y              : Integer;
begin
  Result         := FTables;
  Result.Clear;
  SQLConnection_ := Nil;
  SQLAdapter_    := Nil;
  SQLDataSet_    := Nil;
  if FConnectionString <> '' Then
     Begin
       Try
         SQLConnection_ := SqlConnection.Create(FConnectionString);
         SQLAdapter_    := SqlDataAdapter.Create('SELECT * FROM [sysobjects] WHERE ([type]=''U'') AND ([status] > 0)',SQLConnection_);
         SQLDataSet_    := DataSet.Create;
         SQLAdapter_.Fill(SQLDataSet_, 'sysobjects');
         For Y := 0 To SQLDataSet_.Tables['sysobjects'].Rows.Count-1 do
               Begin
                 Result.Add(SQLDataSet_.Tables['sysobjects'].Rows.Item[Y].Item['name'].ToString);
               End;
         SQLConnection_.Close;
       Except
       End;
       if Assigned(SQLDataSet_)    Then SQLDataSet_.Free;
       if Assigned(SQLAdapter_)    Then SQLAdapter_.Free;
       if Assigned(SQLConnection_) Then SQLConnection_.Free;
     End;
end;

{ TKAADONETConnector }

procedure TKAADONETConnector.DoAfterDelete;
begin
  inherited DoAfterDelete;
  FKASQLAdoNet.DoAfterDelete(FKASQLAdoNet.FADONETConnector);
end;

procedure TKAADONETConnector.DoAfterPost;
begin
  inherited DoAfterPost;
  FKASQLAdoNet.DoAfterPost(FKASQLAdoNet.FADONETConnector);
end;

end.
